@@ -4,7 +4,7 @@
  * various directory search drivers.  It includes functions for searching,
  * adding, removing, and modifying directory entries.
  *
- * $Horde: turba/lib/Driver.php,v 1.57.2.27 2006/05/25 18:16:42 karsten Exp $
+ * $Horde: turba/lib/Driver.php,v 1.57.2.37 2007/03/14 15:58:05 jan Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@csh.rit.edu>
@@ -676,40 +676,63 @@ class Turba_Driver {
     /**
      * Exports a given Turba_Object as an iCalendar vCard.
      *
-     * @param Turba_Object $object  A Turba_Object.
+     * @param Turba_Object $object    A Turba_Object.
+     * @param string       $version   The vcard version to produce.
      *
      * @return Horde_iCalendar_vcard  A Horde_iCalendar_vcard object.
      */
-    function tovCard($object)
+    function tovCard($object, $version = '2.1')
     {
         require_once 'Horde/iCalendar/vcard.php';
+        require_once 'Horde/MIME.php';
 
         $hash = $object->getAttributes();
-        $vcard = &new Horde_iCalendar_vcard();
+        $vcard = &new Horde_iCalendar_vcard($version);
+        $formattedname = false;
+        $charset = $version == '2.1' ? array('CHARSET' => NLS::getCharset()) : array();
 
         foreach ($hash as $key => $val) {
+            if ($version != '2.1') {
+                $val = String::convertCharset($val, NLS::getCharset(), 'utf-8');
+            }
+
             switch ($key) {
             case 'name':
-                $vcard->setAttribute('FN', $val);
+                $vcard->setAttribute('FN', $val, MIME::is8bit($val) ? $charset : array());
+                $formattedname = true;
                 break;
             case 'nickname':
             case 'alias':
-                $vcard->setAttribute('NICKNAME', $val);
+                $vcard->setAttribute('NICKNAME', $val, MIME::is8bit($val) ? $charset : array());
                 break;
             case 'homePhone':
-                $vcard->setAttribute('TEL', $val, array('TYPE'=>'HOME'));
+                if ($version == '2.1') {
+                    $vcard->setAttribute('TEL', $val, array('HOME' => null));
+                } else {
+                    $vcard->setAttribute('TEL', $val, array('TYPE'=>'HOME'));
+                }
                 break;
-
             case 'workPhone':
-                $vcard->setAttribute('TEL', $val, array('TYPE'=>'WORK'));
+                if ($version == '2.1') {
+                    $vcard->setAttribute('TEL', $val, array('WORK' => null));
+                } else {
+                    $vcard->setAttribute('TEL', $val, array('TYPE'=>'WORK'));
+                }
                 break;
-
             case 'cellPhone':
-                $vcard->setAttribute('TEL', $val, array('TYPE'=>'CELL'));
+                if ($version == '2.1') {
+                    $vcard->setAttribute('TEL', $val, array('CELL' => null));
+                } else {
+                    $vcard->setAttribute('TEL', $val, array('TYPE'=>'CELL'));
+                }
                 break;
 
             case 'fax':
-                $vcard->setAttribute('TEL', $val, array('TYPE'=>'FAX'));
+                if ($version == '2.1') {
+                    $vcard->setAttribute('TEL', $val, array('FAX' => null));
+                } else {
+                    $vcard->setAttribute('TEL', $val, array('TYPE'=>'FAX'));
+                }
                 break;
 
             case 'email':
@@ -717,15 +740,11 @@ class Turba_Driver {
                 break;
 
             case 'title':
-                $vcard->setAttribute('TITLE', $val);
-                break;
-
-            case 'company':
-                $vcard->setAttribute('ORG', $val);
+                $vcard->setAttribute('TITLE', $val, MIME::is8bit($val) ? $charset : array());
                 break;
 
             case 'notes':
-                $vcard->setAttribute('NOTE', $val);
+                $vcard->setAttribute('NOTE', $val, MIME::is8bit($val) ? $charset : array());
                 break;
 
             case 'website':
@@ -738,17 +757,57 @@ class Turba_Driver {
             }
         }
 
-        if (!empty($hash['lastname']) || !empty($hash['firstname'])) {
-            $a = array(
-                VCARD_N_FAMILY => isset($hash['lastname']) ? $hash['lastname'] : '',
-                VCARD_N_GIVEN  => isset($hash['firstname']) ? $hash['firstname'] : '',
-                VCARD_N_ADDL   => '',
-                VCARD_N_PREFIX => '',
-                VCARD_N_SUFFIX => '',
-            );
-            $vcard->setAttribute('N', implode(';', $a), array(), false, $a);
+        // No explicit firstname/lastname in data source: we have to guess.
+        if (!isset($hash['lastname'])) {
+            $i = strpos($hash['name'], ',');
+            if (is_int($i)) {
+                // Assume Last, First
+                $hash['lastname'] = String::substr($hash['name'], 0, $i);
+                $hash['firstname'] = trim(String::substr($hash['name'], $i + 1));
+            } elseif (is_int(strpos($hash['name'], ' '))) {
+                // Assume everything after last space as lastname
+                $i = strrpos($hash['name'], ' ');
+                $hash['lastname'] = trim(String::substr($hash['name'], $i + 1));
+                $hash['firstname'] = String::substr($hash['name'], 0, $i);
+            } else {
+                $hash['lastname'] = $hash['name'];
+                $hash['firstname'] = '';
+            }
         }
 
+        $a = array(
+            VCARD_N_FAMILY => isset($hash['lastname']) ? $hash['lastname'] : '',
+            VCARD_N_GIVEN  => isset($hash['firstname']) ? $hash['firstname'] : '',
+            VCARD_N_ADDL   => isset($hash['initials']) ? $hash['initials'] : '',
+            VCARD_N_PREFIX => isset($hash['salutation']) ? $hash['salutation'] : '',
+            VCARD_N_SUFFIX => '',
+        );
+        $val = implode(';', $a);
+        if ($version != '2.1') {
+            $val = String::convertCharset($val, NLS::getCharset(), 'utf-8');
+            $a = String::convertCharset($a, NLS::getCharset(), 'utf-8');
+        }
+        $vcard->setAttribute('N', $val, MIME::is8bit($val) ? $charset : array(), false, $a);
+
+        if (!$formattedname) {
+            $val = empty($hash['firstname']) ? $hash['lastname'] : $hash['firstname'] . ' ' . $hash['lastname'];
+            $vcard->setAttribute('FN', $val, MIME::is8bit($val) ? $charset : array());
+        }
+
+        $org = array();
+        if (isset($hash['company'])) {
+            $org[] = $hash['company'];
+        }
+        if (isset($hash['department'])) {
+            $org[] = $hash['department'];
+        }
+        $val = implode(';', $org);
+        if ($version != '2.1') {
+            $val = String::convertCharset($val, NLS::getCharset(), 'utf-8');
+            $org = String::convertCharset($org, NLS::getCharset(), 'utf-8');
+        }
+        $vcard->setAttribute('ORG', $val, MIME::is8bit($val) ? $charset : array(), false, $org);
+
         /* We can't know if this particular turba source uses a single Address
          * field or multiple for street/city/province/postcode/country. Try to
          * deal with both. */
@@ -764,7 +823,19 @@ class Turba_Driver {
             VCARD_ADR_POSTCODE => isset($hash['homePostalCode']) ? $hash['homePostalCode'] : '',
             VCARD_ADR_COUNTRY  => isset($hash['homeCountry']) ? $hash['homeCountry'] : '',
         );
-        $vcard->setAttribute('ADR', implode(';', $a), array('TYPE' => 'HOME'), true, $a);
+
+        $val = implode(';', $a);
+        if ($version == '2.1') {
+            $params = array('HOME' => null);
+            if (MIME::is8bit($val)) {
+                $params['CHARSET'] = NLS::getCharset();
+            }
+        } else {
+            $params = array('TYPE' => 'HOME');
+            $val = String::convertCharset($val, NLS::getCharset(), 'utf-8');
+            $a = String::convertCharset($a, NLS::getCharset(), 'utf-8');
+        }
+        $vcard->setAttribute('ADR', $val, $params, true, $a);
 
         if (isset($hash['workAddress']) && !isset($hash['workStreet'])) {
             $hash['workStreet'] = $hash['workAddress'];
@@ -778,7 +849,19 @@ class Turba_Driver {
             VCARD_ADR_POSTCODE => isset($hash['workPostalCode']) ? $hash['workPostalCode'] : '',
             VCARD_ADR_COUNTRY  => isset($hash['workCountry']) ? $hash['workCountry'] : '',
         );
-        $vcard->setAttribute('ADR', implode(';', $a), array('TYPE' => 'WORK'), true, $a);
+
+        $val = implode(';', $a);
+        if ($version == '2.1') {
+            $params = array('WORK' => null);
+            if (MIME::is8bit($val)) {
+                $params['CHARSET'] = NLS::getCharset();
+            }
+        } else {
+            $params = array('TYPE' => 'WORK');
+            $val = String::convertCharset($val, NLS::getCharset(), 'utf-8');
+            $a = String::convertCharset($a, NLS::getCharset(), 'utf-8');
+        }
+        $vcard->setAttribute('ADR', $val, $params, true, $a);
 
         return $vcard;
     }
@@ -809,7 +892,9 @@ class Turba_Driver {
 
             switch ($item['name']) {
             case 'FN':
-                $hash['name'] = $item['value'];
+                if (isset($this->fields['name'])) {
+                    $hash['name'] = $item['value'];
+                }
                 break;
 
             case 'N':
@@ -914,12 +999,12 @@ class Turba_Driver {
                         }
                     }
                 } else {
-                    if (isset($item['params']['HOME'])) {
+                    if (isset($item['params']['CELL'])) {
+                        $hash['cellPhone'] = $item['value'];
+                    } elseif (isset($item['params']['HOME'])) {
                         $hash['homePhone'] = $item['value'];
                     } elseif (isset($item['params']['WORK'])) {
                         $hash['workPhone'] = $item['value'];
-                    } elseif (isset($item['params']['CELL'])) {
-                        $hash['cellPhone'] = $item['value'];
                     } else {
                         $hash['homePhone'] = $item['value'];
                     }
@@ -937,12 +1022,11 @@ class Turba_Driver {
                 break;
 
             case 'ORG':
-                $hash['company'] = $item['value'];
-                // According to rfc 2426 ORG is "a single structured text
-                // value consisting of components separated the SEMI-COLON
-                // character (ASCII decimal 59)". We save only one value,
-                // so ignore trailing ";"
-                $hash['company'] = preg_replace('/;$/', '', $hash['company']);
+                // The VCARD 2.1 specification requires the presence of two
+                // SEMI-COLON separated fields: Organizational Name and
+                // Organizational Unit. Additional fields are optional.
+                $hash['company'] = !empty($item['values'][0]) ? $item['values'][0] : '';
+                $hash['department'] = !empty($item['values'][1]) ? $item['values'][1] : '';
                 break;
 
             case 'NOTE':
@@ -959,6 +1043,17 @@ class Turba_Driver {
             }
         }
 
+        /* Ensure we have a valid name field. */
+        if (isset($this->fields['name']) && empty($hash['name'])) {
+            $hash['name'] = $hash['firstname'] . ' ' . $hash['lastname'];
+            if (trim($hash['name']) === '') {
+                $hash['name'] = $vcard->getAttributeDefault('FN');
+            }
+        }
+        if (isset($this->fields['lastname']) && empty($hash['lastname'])) {
+            $hash['lastname'] = $vcard->getAttributeDefault('FN');
+        }
+
         return $hash;
     }
 
