@@ -8,7 +8,7 @@ define('TURBA_VFS_PATH', '.horde/turba/documents');
 /**
  * Turba Base Class.
  *
- * $Horde: turba/lib/Turba.php,v 1.59.4.24 2006/06/26 05:29:16 selsky Exp $
+ * $Horde: turba/lib/Turba.php,v 1.59.4.28 2007/02/26 14:23:13 jan Exp $
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@horde.org>
@@ -66,10 +66,10 @@ class Turba {
                 if (!is_a($addresses, 'PEAR_Error')) {
                     $mail_link = $addresses[$i . ':' . $j];
                     if (is_a($mail_link, 'PEAR_Error')) {
-                        $mail_link = 'mailto:' . $address;
+                        $mail_link = 'mailto:' . urlencode($email_val);
                     }
                 } else {
-                    $mail_link = 'mailto:' . $address;
+                    $mail_link = 'mailto:' . urlencode($email_val);
                 }
 
                 $email_value = Horde::link($mail_link, $email_val) . htmlspecialchars($email_val) . '</a>';
@@ -440,7 +440,7 @@ class Turba {
     {
         // We need to know what the source type is for this share.
         if (empty($params['sourceType'])) {
-            $share = PEAR::raiseError(sprintf(_("Unable to create new share. Missing source type.")));
+            $share = PEAR::raiseError(sprintf('Unable to create a new address book. Enter a source name in Turba\'s configuration at Administration->Setup->Address Book->Shares.'));
             return $share;
         }
 
